using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

[System.Serializable]
public struct QuestionResponse
{
    [SerializeField]
    public string label;

    [SerializeField]
    public bool isCorrect;
}

[System.Serializable]
public struct Question
{
    [SerializeField]
    public string label;

    [SerializeField]
    public List<QuestionResponse> responses;
}

static class RandomExtensions
{
    public static void Shuffle<T>(T[] array)
    {
        int n = array.Length;
        while (n > 1)
        {
            int k = Random.Range(0, n--);
            T temp = array[n];
            array[n] = array[k];
            array[k] = temp;
        }
    }
}

public class BossManager : MonoBehaviour
{
    [SerializeField]
    public List<Question> questions;
    public float timerInSeconds = 15;

    public HorizontalLayoutGroup buttonsUpperRow;
    public HorizontalLayoutGroup buttonsBottomRow;
    public Text questionText;
    public BossFightParticipant player;
    public BossFightParticipant boss;

    public string nextLevelName;
    public string previousLevelName;

    public GameObject questionsPanel;
    public GameObject winPanel;
    public GameObject losePanel;
    public Text timerText;
    public GameObject mapPanel;

    public int damagePercentageAmount = 20;

    private Question[] questionsRandomized;
    private int currentQuestionIndex = -1;
    private float remainingTime;
    private bool isEnd = false;

    AudioSource audioSource;
    public AudioClip correctAudio;
    public AudioClip wrongClip;
    public AudioClip winAudio;
    public AudioClip lostAudio;

    void Start()
    {
        audioSource = GetComponent<AudioSource>();

        questionsPanel.SetActive(true);
        winPanel.SetActive(false);
        losePanel.SetActive(false);
        mapPanel.SetActive(false);

        questionsRandomized = questions.ToArray();
        RandomExtensions.Shuffle(questionsRandomized);
        ShowNextQuestion();
    }

    void ShowNextQuestion() {
        currentQuestionIndex++;

        if (currentQuestionIndex >= questionsRandomized.Length)
        {
            Debug.LogError("No more questions!");
            return;
        }

        Question question = questionsRandomized[currentQuestionIndex];
        questionText.text = question.label;

        List<QuestionResponse> responses = new List<QuestionResponse>(question.responses);
        List<QuestionResponse> selectedResponses = new List<QuestionResponse>();
        int correctResponseIndex = responses.FindIndex(r => r.isCorrect);
        if(correctResponseIndex == -1)
        {
            Debug.LogError($"No correct response for question '{question.label}' was found");
            return;
        }

        selectedResponses.Add(responses[correctResponseIndex]);
        responses.RemoveAt(correctResponseIndex);

        for(int i = 0; i < 3; i++)
        {
            int randomIndex = Random.Range(0, responses.Count);
            selectedResponses.Add(responses[randomIndex]);
            responses.RemoveAt(randomIndex);
        }

        QuestionResponse[] selectedResponsesArray = selectedResponses.ToArray();
        RandomExtensions.Shuffle(selectedResponsesArray);

        SetButtons(selectedResponsesArray);
        remainingTime = timerInSeconds;
    }

    private void Update()
    {
        if (isEnd)
        {
            return;
        }

        remainingTime -= Time.deltaTime;
        if(remainingTime <= 0)
        {
            timerText.text = "0";

            UnsubscribeButtons();
            if (player.GetHit(damagePercentageAmount))
            {
                PlayerLose();
            }
            else
            {
                ShowNextQuestion();
            }
        } else
        {
            timerText.text = Mathf.Floor(remainingTime).ToString();
        }
    }

    void SetButtons(QuestionResponse[] responses)
    {
        for (int i = 0; i < responses.Length; i++)
        {
            QuestionResponse response = responses[i]; 
            Button btn;
            if (i < 2)
            {
                btn = buttonsUpperRow.transform.GetChild(i).GetComponent<Button>();
            }
            else
            {
                btn = buttonsBottomRow.transform.GetChild(i - 2).GetComponent<Button>();
            }

            Text btnText = btn.GetComponentInChildren<Text>();
            btnText.text = response.label;

            btn.onClick.AddListener(() => ResponseSelected(response));
        }
    }

    void UnsubscribeButtons()
    {
        for (int i = 0; i < buttonsUpperRow.transform.childCount; i++)
        {
            Button btn = buttonsUpperRow.transform.GetChild(i).GetComponent<Button>();
            btn.onClick.RemoveAllListeners();
        }

        for (int i = 0; i < buttonsBottomRow.transform.childCount; i++)
        {
            Button btn = buttonsBottomRow.transform.GetChild(i).GetComponent<Button>();
            btn.onClick.RemoveAllListeners();
        }
    }

    void ResponseSelected(QuestionResponse response)
    {
        UnsubscribeButtons();
        GUI.FocusControl(null);

        string correctString = response.isCorrect ? "correct" : "incorrect";
        Debug.Log($"{response.label} => {correctString}" );

        if(response.isCorrect)
        {
            if (audioSource && correctAudio)
            {
                audioSource.PlayOneShot(correctAudio);
            }

            if (boss.GetHit(damagePercentageAmount))
            {
                PlayerWin();
            }
        } else
        {
            if (audioSource && wrongClip)
            {
                audioSource.PlayOneShot(wrongClip);
            }

            if (player.GetHit(damagePercentageAmount))
            {
                PlayerLose();
            }
        }

        ShowNextQuestion();
    }

    void PlayerWin()
    {
        isEnd = true;

        questionsPanel.SetActive(false);
        winPanel.SetActive(true);
        StartCoroutine(LoadLevelWithMap(4f, 4f, nextLevelName));

        if(audioSource && winAudio)
        {
            audioSource.PlayOneShot(winAudio);
        }
    }

    void PlayerLose()
    {
        isEnd = true;

        questionsPanel.SetActive(false);
        losePanel.SetActive(true);
        StartCoroutine(LoadLevel(4f, previousLevelName));

        if (audioSource && lostAudio)
        {
            audioSource.PlayOneShot(lostAudio);
        }
    }

    IEnumerator LoadLevel(float loadDelay, string levelName)
    {
        yield return new WaitForSeconds(loadDelay);
        SceneManager.LoadScene(levelName, LoadSceneMode.Single);
    }

    IEnumerator LoadLevelWithMap(float mapDelay, float loadDelay, string levelName)
    {
        yield return new WaitForSeconds(mapDelay);
        mapPanel.SetActive(true);
        yield return new WaitForSeconds(loadDelay);
        SceneManager.LoadScene(levelName, LoadSceneMode.Single);
    }
}
